// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-enterprise-management/pb"
	"strings"
	"google.golang.org/grpc/status"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"smart-enterprise-management/utils"
	"smart-enterprise-management/parser"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
	"github.com/jaryhe/gopkgs/util"
)

func getB64Pass(passwd string) (string, error) {
	cryPasswd, err := util.AesEncrypt(passwd, CRYPTO_KEY)
	if err != nil {
		return "", errors.ParamsError
	}
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret, nil
}

func checkProjectAddParam(req *v1.ProjectAddRequest) error {
	limitAreaCode := strings.TrimRight(parser.Conf.CityAreaCode, "0")
	switch {
	case req.CompanyId < 1:
		return status.Error(10003, "未知公司")
	case req.Price <= 0 || req.Price > 9999999:
		return status.Error(10003, "总投资错误")
	case req.Name == "":
		return status.Error(10003, "项目名称不能为空")
	case req.Category == 0:
		return status.Error(10003, "项目类别不能为空")
	case req.ConstructType == 0:
		return status.Error(10003, "项目建设性质不能为空")
	case req.CoveredArea == 0:
		return status.Error(10003, "建筑面积不能为空")
	case req.StartDay == 0:
		return status.Error(10003, "计划开工日期不能为空")
	case req.EndDay == 0:
		return status.Error(10003, "计划竣工日期不能为空")
	case req.EndDay <= req.StartDay:
		return status.Error(10003, "计划竣工日期不能早于计划开工日期")
	case req.SafetyRecordNo == "":
		return status.Error(10003, "安监备案号不能为空")
	case req.InvestType == 0:
		return status.Error(10003, "资金来源不能为空")
	case req.AreaCode == "":
		return status.Error(10003, "行政区域代码不能为空")
	case req.Location == "":
		return status.Error(10003, "建设地址不能为空")
	case req.Lon <= 0 || req.Lat <= 0 :
		return status.Error(10003, "经纬度不能为空")
	case req.Scale == "":
		return status.Error(10003, "项目规模不能为空")
	case req.ProjectStatus < consts.ProjectStatusPrepare || req.ProjectStatus > consts.ProjectStatusStop:
		return status.Error(10003, "不支持的项目状态")

	case len(req.AreaCode) != 6 ||
			(parser.Conf.CityAreaCode != "" && strings.Contains(req.AreaCode, limitAreaCode) == false):
		return status.Error(10003, fmt.Sprintf("区域代码错误"))
	}


	return nil
}


func getProjectCodeFromProvincial(name string, areaCode string, category int32) (string, error) {
	mreq := v1.ProvincialGetProjectCodeRequest{}
	mreq.Name = name
	mreq.AreaCode = areaCode
	mreq.Category = category
	mreply, err := pb.Provincial.ProvincialGetProjectCode(context.Background(), &mreq)
	if err != nil {
		return "", err
	}
	return mreply.Code, nil
}

func postProjectToProvincial(company model.TCompany, project model.ProjectInfo) (string, error) {
		mreq := v1.ProvincialPostProjectRequest{
		ProjectCode:project.Code,
		ProjectName:project.Name,
		ProjectCategory:consts.ProjectCategoryM[project.Category],
		OwnerName:company.Name,
		BuildCorporationCode:company.SocialCode,
		AreaCode:project.Areacode,
		Address:project.Address,
		Invest:fmt.Sprintf("%f", project.Invest),
		PrjSize:project.Scale,
		StartDate:project.Startdate.Format("2006-01-02"),
		CompleteDate:project.Enddate.Format("2006-01-02"),
		PrjStatus:consts.ProjectStatusM[project.Prjstatus],
		ConstructType:int32(project.Constructtype),
		InvestType:project.InvestType,
		BuildingArea:project.Buildingarea,
		Scale:project.Scale,
	}
	mreply, err := pb.Provincial.ProvincialPostProject(context.Background(), &mreq)
	if err != nil {
		return "", err
	}

	return mreply.Id, nil
}

func generateProjectCode(name string, areaCode string, category int32) (string, error){
	// TODO 实际会调省厅接口获取
	if parser.Conf.UseProvincial {
		ret, err := getProjectCodeFromProvincial(name, areaCode, category)
		if err != nil {
			return "", errors.ProvincialFailed
		}
		return ret, nil
	}


	now := time.Now().Format("2006-01-02")
	prefix := areaCode + string(now[2:4])+string(now[5:7])+string(now[8:])
	ret, err := model.GetNextProjectCode(database.DB(), prefix)
	if err != nil {
		return "", errors.DataBaseError
	}
	return ret, nil
}




func ProjectAdd(ctx context.Context, req *v1.ProjectAddRequest) (reply *v1.ProjectAddReply, err error) {
	if err := checkProjectAddParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	company := model.TCompany{}
	where := map[string]interface{}{
		"ID":req.CompanyId,
	}
	err = company.Find(database.DB(), where, nil)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if company.Id == 0 {
		return nil, errors.NoRecordError
	}
	reply = &v1.ProjectAddReply{}
	projectCode := ""
	addTask := func(db *gorm.DB) error {
		p := model.ProjectInfo{}
		where := map[string]interface{}{
			"SafetyNo":          req.SafetyRecordNo,
			"Name":req.Name,
		}
		err = p.FindOr(db, nil, where)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.ID > 0 {
			return errors.ProjectExistError
		}

		p = model.ProjectInfo{}
		where = map[string]interface{}{
			"Lng":  req.Lon,
			"Lat":	req.Lat,
		}
		err = p.FindOr(db, where, nil)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.ID > 0 {
			return status.Error(20004, fmt.Sprintf("坐标%f,%f上已配置工地", req.Lon, req.Lat))
		}

		p.Code, err = generateProjectCode(req.Name, req.AreaCode, int32(req.Category))
		if err != nil {
			return err
		}
		projectCode = p.Code
		p.Cid = req.CompanyId
		//p.ProjectNo = req.ProjectNo
		p.Name = req.Name

		p.Safetyno = req.SafetyRecordNo
		p.Address = req.Location
		p.Name = req.Name
		p.Areacode = req.AreaCode
		p.Buildingarea = req.CoveredArea
		p.Category = req.Category
		p.Constructtype = req.ConstructType
		p.InvestType = req.InvestType
		p.Invest = req.Price
		p.Scale = req.Scale
		p.Prjstatus = req.ProjectStatus
		p.Description = req.Desc
		p.Lat = req.Lat
		p.Lng = req.Lon

		if len(req.ProjectPlan) > 0 {
			bytes, _ := json.Marshal(req.ProjectPlan)
			p.Planpic = string(bytes)
		}
		if len(req.ProjectEffectPic) > 0 {
			bytes, _ := json.Marshal(req.ProjectEffectPic)
			p.Effectpic = string(bytes)
		}

		p.Updatedat = now
		p.Createdat = now
		p.Startdate = time.Unix(req.StartDay, 0)
		p.Enddate = time.Unix(req.EndDay, 0)

		// TODO 正式需要向省厅发数据
		if parser.Conf.UseProvincial {
			provincialId, err := postProjectToProvincial(company, p)
			if err != nil {
				logger.Error("postProjectToProvincial",
					zap.String("err", err.Error()))
				return errors.ProvincialFailed
			}
			p.ProvincialVerifyID = provincialId
		}

		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		reply.Id = p.ID
		return nil
	}

	createProjectUserTask := func(db *gorm.DB) error {
		// 建设单位账号
		p := model.TSiteUser{
			Username:"jsdw"+fmt.Sprintf("%d", reply.Id),
			//Username:"c"+utils.GenerateRandomStr(3, "mix")+fmt.Sprintf("%d", reply.Id)+utils.GenerateRandomStr(3, "char"),
			UserType:consts.ProjectUserTypeConstruction,
			CreatedAt:now,
			UpdatedAt:now,
			Enable:true,
			ProjectId:reply.Id,
			SiteUser:0,
		}
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 项目部账号
		p.Id = 0
		p.Username = "xmb"+fmt.Sprintf("%d", reply.Id)
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		p.SiteUser = 1
		p.UserType = consts.ProjectUserTypeProject
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if parser.Conf.RunMode == "dev" {
			p.Id = 0
			p.Username = "admin"+fmt.Sprintf("%d", reply.Id)
			p.Passwd,_ = getB64Pass("123456")
			p.SiteUser = 1
			p.UserType = consts.ProjectUserTypeProject
			err = p.Insert(db)
			if err != nil {
				logger.Error("ProjectAdd",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}

		// 监理部账号
		p.Id = 0
		p.Username = "jlb"+fmt.Sprintf("%d", reply.Id)
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		p.SiteUser = 0
		p.UserType = consts.ProjectUserTypeSupervision
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		return nil
	}

	createCorpTask := func(db *gorm.DB) error {
		return nil
		for i := 1; i < 12; i++ {
			p := model.ProjectCorpInfo{}
			p.ProjectId = reply.Id
			p.CorpType = uint32(i)
			if p.CorpType == consts.ProjectCorpConstruction {
				p.CorpCode = company.SocialCode
				p.CorpName = company.Name
			}
			p.Code = projectCode
			err := p.Insert(db)
			if err != nil {
				return errors.DataBaseError
			}
		}

		return nil
	}
	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(addTask),
		transtasker.NewTask(createProjectUserTask),transtasker.NewTask(createCorpTask))

	return reply, err


}
