// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
)

var districtCache = []*v1.ProjectDistrictProvince{}

func ProjectDistrict(ctx context.Context, req *v1.ProjectDistrictRequest) (reply *v1.ProjectDistrictReply, err error) {
	p := model.TDistrict{}

	reply = &v1.ProjectDistrictReply{}
	if len(districtCache) > 0 {
		reply.List = districtCache
		return reply, nil
	}
	// 获取第一级地址
	level1List, err := p.List(database.DB(), []int64{0})
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectDistrict",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 获取第二级地址
	ids := make([]int64, len(level1List))
	for i, v := range level1List {
		ids[i] = v.Id
	}
	level2List, err := p.List(database.DB(), ids)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectDistrict",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 获取第三级地址
	ids = make([]int64, len(level2List))
	for i, v := range level2List {
		ids[i] = v.Id
	}
	level3List, err := p.List(database.DB(), ids)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectDistrict",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	// 根据一级id构造一级地址map
	m1 := map[int64]*v1.ProjectDistrictProvince{}
	for _, v := range level1List {
		item := &v1.ProjectDistrictProvince{
			Id:   v.Id,
			Addr: v.Name,
		}
		m1[v.Id] = item
	}

	// 根据二级id构造三级地址map
	m3 := map[int64][]*v1.ProjectDistrictZone{}
	for _, v := range level3List {
		item := &v1.ProjectDistrictZone{
			Id:   v.Id,
			Addr: v.Name,
		}
		m3[v.ParentId] = append(m3[v.ParentId], item)
	}

	// 根据一级id构造二级地址map
	// 如果三级地址为空则将其移至三级地址
	m2 := map[int64][]*v1.ProjectDistrictCity{}
	for _, v := range level2List {
		if _, ok := m3[v.Id]; !ok {
			item := &v1.ProjectDistrictZone{
				Id:   v.Id,
				Addr: v.Name,
			}
			m3[v.ParentId] = append(m3[v.ParentId], item)
			continue
		}
		item := &v1.ProjectDistrictCity{
			Id:   v.Id,
			Addr: v.Name,
			List: m3[v.Id],
		}
		m2[v.ParentId] = append(m2[v.ParentId], item)
	}

	// 如果二级地址下的三级地址为空将二级地址的父地址copy为二级地址
	// 如北京-北京-东城区
	for _, v := range level2List {
		if _, ok := m3[v.Id]; !ok  {
			if _, exist := m2[v.ParentId]; !exist {
				item := &v1.ProjectDistrictCity{
					Id:   m1[v.ParentId].Id,
					Addr: m1[v.ParentId].Addr,
					List: m3[v.ParentId],
				}
				m2[v.ParentId] = append(m2[v.ParentId], item)
			}

		}

	}

	// 构造返回结果
	reply.List = make([]*v1.ProjectDistrictProvince, len(level1List))
	for i, v := range level1List {
		reply.List[i] = &v1.ProjectDistrictProvince{
			Id:   v.Id,
			Addr: v.Name,
			List: m2[v.Id],
		}
	}
	districtCache = reply.List
	return reply, nil
}
