// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
	"smart-enterprise-management/parser"
	"strings"
	"fmt"
)

func checkProjectUpdateParam(req *v1.ProjectUpdateRequest) error {
	limitAreaCode := strings.TrimRight(parser.Conf.CityAreaCode, "0")
	switch {
	case req.Id < 1:
		return errors.ParamsError
	case req.Price <= 0 || req.Price > 9999999:
		return status.Error(10003, "总投资错误")
	case req.Name == "":
		return status.Error(10003, "项目名称不能为空")
	case req.Category == 0:
		return status.Error(10003, "项目类别不能为空")
	case req.ConstructType == 0:
		return status.Error(10003, "项目建设性质不能为空")
	case req.CoveredArea == 0:
		return status.Error(10003, "建筑面积不能为空")
	case req.StartDay == 0:
		return status.Error(10003, "计划开工日期不能为空")
	case req.EndDay == 0:
		return status.Error(10003, "计划竣工日期不能为空")
	case req.EndDay <= req.StartDay:
		return status.Error(10003, "计划竣工日期不能早于计划开工日期")
	case req.SafetyRecordNo == "":
		return status.Error(10003, "安监备案号不能为空")
	case req.InvestType == 0:
		return status.Error(10003, "资金来源不能为空")

	case req.Location == "":
		return status.Error(10003, "建设地址不能为空")
	case req.Lon <= 0 || req.Lat <= 0 :
		return status.Error(10003, "经纬度不能为空")
	case req.Scale == "":
		return status.Error(10003, "项目规模不能为空")
	case req.ProjectStatus < consts.ProjectStatusPrepare || req.ProjectStatus > consts.ProjectStatusStop:
		return status.Error(10003, "不支持的项目状态")
	case req.AreaCode != "" && (len(req.AreaCode) != 6 || (parser.Conf.CityAreaCode != "" && strings.Contains(req.AreaCode, limitAreaCode) == false)):
		return status.Error(10003, fmt.Sprintf("区域代码错误"))
	}
	return nil
}

func ProjectUpdate(ctx context.Context, req *v1.ProjectUpdateRequest) (reply *v1.ProjectUpdateReply, err error) {
	if err := checkProjectUpdateParam(req); err != nil {
		return nil, err
	}
	reply = &v1.ProjectUpdateReply{}
	updateTask := func(db *gorm.DB) error {
		p := model.ProjectInfo{}
		and := map[string]interface{}{
			"ID !=":req.Id,
			//"safety_record_no":          req.SafetyRecordNo,
		}
		or := map[string]interface{}{
			"SafetyNo":          req.SafetyRecordNo,
			"Name":req.Name,
		}
		err = p.FindOr(database.DB(), and, or)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.ID > 0 {
			return errors.ProjectExistError
		}
		p = model.ProjectInfo{}
		and = map[string]interface{}{
			"ID":req.Id,
		}
		err = p.FindOr(database.DB(), and, nil)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.ID == 0 {
			return errors.ProjectNotExistError
		}

		m := map[string]interface{}{
			"Name":req.Name,
			"SafetyNo":req.SafetyRecordNo,
			"Address":req.Location,
			"BuildingArea":req.CoveredArea,
			"Category":req.Category,
			"ConstructType":req.ConstructType,
			"InvestType":req.InvestType,
			"Invest":req.Price,
			"Scale":req.Scale,
			"Description":req.Desc,
			"PrjStatus":req.ProjectStatus,
			"Lat":req.Lat,
			"Lng":req.Lon,
			"StartDate": time.Unix(req.StartDay, 0),
			"EndDate":time.Unix(req.EndDay, 0),
			"UpdatedAt":time.Now(),
			"PlanPic":"",
			"EffectPic":"",
		}
		if req.AreaCode != "" {
			m["AreaCode"] = req.AreaCode
		}




		if len(req.ProjectPlan) > 0 {
			bytes, _ := json.Marshal(req.ProjectPlan)
			m["PlanPic"] = string(bytes)
		}
		if len(req.ProjectEffectPic) > 0 {
			bytes, _ := json.Marshal(req.ProjectEffectPic)
			m["EffectPic"] = string(bytes)
		}

		p = model.ProjectInfo{}
		and = map[string]interface{}{
			"ID":req.Id,
		}

		// 省厅数据
		if parser.Conf.UseProvincial {
			if p.ProvincialVerifyStatus != 204 {
				return errors.ProvincialVerifyError
			}
			if p.ProvincialVerifyCode != 0 {
				// 重新上报项目信息
				company := &model.TCompany{}
				where := map[string]interface{}{
					"ID":p.Cid,
				}
				err := company.Find(database.DB(), where, nil)
				if err != nil {
					return errors.DataBaseError
				}
				project := model.ProjectInfo{
					Name:p.Name,
					Code:p.Code,
					Category:req.Category,
					Constructtype:req.ConstructType,
					InvestType:req.InvestType,
					Buildingarea:req.CoveredArea,
					Areacode:req.AreaCode,
					Address:req.Location,
					Invest:req.Price,
					Scale:req.Scale,
					Startdate:time.Unix(req.StartDay, 0),
				}
				provincialId, err := postProjectToProvincial(*company, project)
				if err != nil {
					return errors.ProvincialFailed
				}
				m["ProvincialVerifyID"] = provincialId
				m["ProvincialVerifyStatus"] = 0
				m["ProvincialVerifyCode"] = 0
			}
		}

		err := p.Update(database.DB(), and, m)
		if err != nil {
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		return nil
	}


	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(updateTask))

	return reply, err


}
